// Maria transition qualifier expression class -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "TransitionQualifier.h"
#include "Net.h"
#include "BoolType.h"
#include "Transition.h"
#include "Printer.h"

/** @file TransitionQualifier.C
 * Transition instance qualifier
 */

TransitionQualifier::TransitionQualifier (const class Transition& transition,
					  class Expression& expr) :
  Expression (),
  myTransition (&transition), myExpr (&expr)
{
  assert (myExpr->getType ()->getKind () == Type::tBool);
  setType (*myExpr->getType ());
}

TransitionQualifier::~TransitionQualifier ()
{
  myExpr->destroy ();
}

class Expression*
TransitionQualifier::ground (const class Valuation& valuation,
			     class Transition* transition,
			     bool declare)
{
  class Expression* e = myExpr->ground (valuation, transition, declare);
  if (!e)
    return NULL;
  assert (valuation.isOK ());
  if (e == myExpr) {
    e->destroy ();
    return copy ();
  }
  else
    return new class TransitionQualifier (*myTransition, *e);
}

class Expression*
TransitionQualifier::substitute (class Substitution& substitution)
{
  class Expression* e = myExpr->substitute (substitution);
  if (!e)
    return NULL;
  if (e == myExpr) {
    e->destroy ();
    return copy ();
  }
  else
    return new class TransitionQualifier (*myTransition, *e);
}

bool
TransitionQualifier::depends (const class VariableSet& vars,
			      bool complement) const
{
  return myExpr->depends (vars, complement);
}

bool
TransitionQualifier::forVariables (bool (*operation)
				   (const class Expression&,void*),
				   void* data) const
{
  return myExpr->forVariables (operation, data);
}

class Expression*
TransitionQualifier::disqualify (const class Transition& transition)
{
  return myTransition == &transition ? myExpr->copy () : 0;
}

void
TransitionQualifier::display (const class Printer& printer) const
{
  printer.printRaw ("trans");
  printer.delimiter (' ');
  printer.print (myTransition->getName ());
  if (myExpr) {
    printer.delimiter (':');
    myExpr->display (printer);
  }
}
