// State of reachability analysis -*- c++ -*-

#ifndef SEARCH_H_
# define SEARCH_H_
# ifdef __GNUC__
#  pragma interface
# endif // __GNUC__

# include <list>
# include "typedefs.h"

/** @file Search.h
 * List of search states
 */

/** List of search states */
class SearchList
{
public:
  /** List of the state numbers in the search stack or queue */
  typedef std::list<card_t> List;
  /** Iterator to the state number list */
  typedef List::iterator iterator;
  /** Constant iterator to the state number list */
  typedef List::const_iterator const_iterator;

  /** Constructor */
  SearchList () : myList () {}
  /** Copy constructor */
  SearchList (const class SearchList& old) : myList (old.myList) {}
  /** Assignment operator */
  class SearchList& operator= (const class SearchList& old) {
    myList = old.myList;
    return *this;
  }
  /** Destructor */
  ~SearchList () {}

  /** @name Accessors to the state number list */
  /*@{*/
  bool empty () const { return myList.empty (); }
  size_t size () const { return myList.size (); }
  const_iterator begin () const { return myList.begin (); }
  const_iterator end () const { return myList.end (); }
  void clear () { myList.clear (); }
  /*@}*/

  /** Insert an item to the list */
  void push (card_t item) { myList.push_front (item); }
  /** Fetch an item
   * @param tail	flag: retrieve from tail of list instead of head */
  card_t pop (bool tail = false) {
    iterator i;
    if (tail)
      (i = myList.end ())--;
    else
      i = myList.begin ();
    card_t search = *i;
    myList.erase (i);
    return search;
  }

  /** Determine whether the list contains an item
   * @param item	item being sought
   * @return		true if the item is in the list
   */
  bool contains (card_t item) const {
    for (const_iterator i = myList.begin (); i != myList.end (); i++)
      if (*i == item)
	return true;
    return false;
  }

  /** Append a list to this
   * @param other	list to be appended to this
   */
  void append (const class SearchList& other) {
    for (const_iterator i = other.myList.begin ();
	 i != other.myList.end (); i++)
      myList.push_back (*i);
  }

  /** Remove an item from the list
   * @param item	item to be removed
   */
  void remove (card_t item) {
    for (iterator i = myList.begin (); i != myList.end (); i++) {
      if (*i == item) {
	myList.erase (i);
	break;
      }
    }
  }

private:
  /** The search list */
  List myList;
};

#endif // SEARCH_H_
